// ignore_for_file: non_constant_identifier_names

import 'package:flutter/cupertino.dart';
import 'package:pam_flutter/models/consent_message_model.dart';
import 'package:pam_flutter/pam_flutter.dart';
import 'package:pam_flutter/repositories/base_repository.dart';
import 'package:pam_flutter/repositories/types.dart';
import 'package:pam_flutter/types.dart';
import 'package:pam_flutter/utils/time_helper.dart';

class ConsentMessageTypes {
  static String get tracking_type => "tracking_type";

  static String get contacting_type => "contacting_type";
}

class ConsentMessageSettingConsentDetails {
  ConsentMessageSettingConsentDetails._();

  static String get terms_and_conditions => "TermsAndConditions";

  static String get analytics_cookies => "AnalyticsCookies";

  static String get email => "Email";

  static String get facebook_messenger => "FacebookMessenger";

  static String get line => "Line";

  static String get marketing_cookies => "MarketingCookies";

  static String get mobile_notification => "MobileNotification";

  static String get necessary_cookies => "NecessaryCookies";

  static String get preferences_cookies => "PreferencesCookies";

  static String get privacy_overview => "PrivacyOverview";

  static String get sms => "SMS";

  static String get social_media_cookies => "SocialMediaCookies";
}

class PDPARepository extends BaseDataRepository<ConsentMessageModel> {
  final BuildContext buildCtx;
  final IRepositoryOptions options;

  PDPARepository({
    required this.buildCtx,
    required this.options,
  }) : super(buildCtx, options);

  @override
  List<ConsentMessageModel> transforms(tss) {
    return ConsentMessageModel.toList(tss);
  }

  @override
  ConsentMessageModel? transform(ts) {
    return ConsentMessageModel.fromJson(ts);
  }

  void allowConsentSetting(String settingName, bool isAllowed) {
    try {
      this.toLoadingStatus();
      ConsentMessageModel model = this.data!;

      switch (settingName) {
        case "TermsAndConditions":
          model.Setting.TermsAndConditions.IsAllowed = isAllowed;
          super.setInnerData(model.toMap());
          break;
        case "AnalyticsCookies":
          model.Setting.AnalyticsCookies.IsAllowed = isAllowed;
          super.setInnerData(model.toMap());
          break;
        case "Email":
          model.Setting.Email.IsAllowed = isAllowed;
          super.setInnerData(model.toMap());
          break;
        case "FacebookMessenger":
          model.Setting.FacebookMessenger.IsAllowed = isAllowed;
          super.setInnerData(model.toMap());
          break;
        case "Line":
          model.Setting.Line.IsAllowed = isAllowed;
          super.setInnerData(model.toMap());
          break;
        case "MarketingCookies":
          model.Setting.MarketingCookies.IsAllowed = isAllowed;
          super.setInnerData(model.toMap());
          break;
        case "MobileNotification":
          model.Setting.MobileNotification.IsAllowed = isAllowed;
          super.setInnerData(model.toMap());
          break;
        case "NecessaryCookies":
          model.Setting.NecessaryCookies.IsAllowed = isAllowed;
          super.setInnerData(model.toMap());
          break;
        case "PreferencesCookies":
          model.Setting.PreferencesCookies.IsAllowed = isAllowed;
          super.setInnerData(model.toMap());
          break;
        case "PrivacyOverview":
          model.Setting.PrivacyOverview.IsAllowed = isAllowed;
          super.setInnerData(model.toMap());
          break;
        case "SMS":
          model.Setting.SMS.IsAllowed = isAllowed;
          super.setInnerData(model.toMap());
          break;
        case "SocialMediaCookies":
          model.Setting.SocialMediaCookies.IsAllowed = isAllowed;
          super.setInnerData(model.toMap());
          break;
        default:
      }

      this.dataSC.add(this.data);

      this.toLoadedStatus();
    } catch (e) {
      this.toErrorStatus(e);
    }
  }

  void allowAllConsentSetting() {
    try {
      this.toLoadingStatus();
      ConsentMessageModel model = this.data!;

      model.Setting.TermsAndConditions.IsAllowed = true;
      model.Setting.AnalyticsCookies.IsAllowed = true;
      model.Setting.Email.IsAllowed = true;
      model.Setting.FacebookMessenger.IsAllowed = true;
      model.Setting.Line.IsAllowed = true;
      model.Setting.MarketingCookies.IsAllowed = true;
      model.Setting.MobileNotification.IsAllowed = true;
      model.Setting.NecessaryCookies.IsAllowed = true;
      model.Setting.PreferencesCookies.IsAllowed = true;
      model.Setting.PrivacyOverview.IsAllowed = true;
      model.Setting.SMS.IsAllowed = true;
      model.Setting.SocialMediaCookies.IsAllowed = true;
      super.setInnerData(model.toMap());

      this.dataSC.add(this.data);

      this.toLoadedStatus();
    } catch (e) {
      this.toErrorStatus(e);
    }
  }

  Future<void> sendAllowConsentSetting(String id) async {
    try {
      this.toLoadingStatus();

      if (this.data?.Type == ConsentMessageTypes.tracking_type) {
        Pam.trackCustomEvent(eventName: "allow_consent", payload: {
          "_consent_message_id": id,
          "_allow_terms_and_conditions":
              this.data?.Setting.TermsAndConditions.IsAllowed ?? false,
          "_allow_privacy_overview":
              this.data?.Setting.PrivacyOverview.IsAllowed ?? false,
          "_allow_necessary_cookies":
              this.data?.Setting.NecessaryCookies.IsAllowed ?? false,
          "_allow_preferences_cookies":
              this.data?.Setting.PreferencesCookies.IsAllowed ?? false,
          "_allow_analytics_cookies":
              this.data?.Setting.AnalyticsCookies.IsAllowed ?? false,
          "_allow_marketing_cookies":
              this.data?.Setting.MarketingCookies.IsAllowed ?? false,
          "_allow_social_media_cookies":
              this.data?.Setting.SocialMediaCookies.IsAllowed ?? false,
        });
      } else if (this.data?.Type == ConsentMessageTypes.contacting_type) {
        Pam.trackCustomEvent(eventName: "allow_consent", payload: {
          "_consent_message_id": id,
          "_allow_terms_and_conditions":
              this.data?.Setting.TermsAndConditions.IsAllowed ?? false,
          "_allow_privacy_overview":
              this.data?.Setting.TermsAndConditions.IsAllowed ?? false,
          "_allow_email": this.data?.Setting.Email.IsAllowed ?? false,
          "_allow_sms": this.data?.Setting.SMS.IsAllowed ?? false,
          "_allow_line": this.data?.Setting.Line.IsAllowed ?? false,
          "_allow_facebook":
              this.data?.Setting.FacebookMessenger.IsAllowed ?? false,
          "_allow_mobile_notification":
              this.data?.Setting.MobileNotification.IsAllowed ?? false,
        });
      }

      Pam.dispatch(PamStandardCallback.on_consent, {
        "consent_id": this.data?.Id ?? "",
        "consent_allow_id": "",
        "allow_terms_conditions": this.data?.Setting.TermsAndConditions.IsAllowed ?? false,
        "allow_privacy_overview": this.data?.Setting.PrivacyOverview.IsAllowed ?? false,
        "allow_necessary_cookies": this.data?.Setting.NecessaryCookies.IsAllowed ?? false,
        "allow_preferences_cookies": this.data?.Setting.PreferencesCookies.IsAllowed ?? false,
        "allow_analytics_cookies": this.data?.Setting.AnalyticsCookies.IsAllowed ?? false,
        "allow_marketing_cookies": this.data?.Setting.MarketingCookies.IsAllowed ?? false,
        "allow_social_media_cookies": this.data?.Setting.SocialMediaCookies.IsAllowed ?? false,
      });

      await TimeHelper.sleep();

      this.toLoadedStatus();
    } catch (e) {
      this.toErrorStatus(e);
    }
  }
}
